@@ -2,6 +2,15 @@
 #
 # install.tcl - Installation support
 #
+option add *Entry.background		white	startup
+option add *Entry.foreground 		black 	startup
+option add *Button.padX 		1	startup
+option add *Button.padX 		1	startup
+option add *Button.highlightThickness 	0	startup
+option add *Entry.relief 		flat	startup
+option add *Entry.highlightThickness 	0	startup
+option add *font 			fixed 	startup
+
 proc install_init { appName dotFile } {
     global install
     set install(appName) $appName
@@ -13,46 +22,47 @@ proc install_init { appName dotFile } {
 	}
     } else {
 	if {[catch {glob ../$appName*/$dotFile} files] == 0} {
-	    wm withdraw .
-	    toplevel .config
-	    set fm [frame .config.rim -bd 10]
-	    message $fm.msg -aspect 1500 -text \
-"Please select a saved configuration."
-	    global tk_version
-	    if {$tk_version >= 3.3} {
-		pack $fm -fill both -expand 1
-		pack $fm.msg
-	    } else {
-		pack append .config $fm {top fill expand}
-		pack append $fm $fm.msg {top fill expand}
-	    }
-	    set id 0
-	    foreach f $files {
-		button $fm.but$id -text $f -command [list installConfig $f]
-		if {$tk_version >= 3.3} {
-		    pack $fm.but$id -fill both -expand 1
-		} else {
-		    pack append $fm $fm.but$id {top fill expand}
-		}
-		incr id
-	    }
-	    button $fm.but$id -text "Ignore configurations" -command installConfig
-	    if {$tk_version >= 3.3} {
-		pack $fm.but$id -fill both -expand 1
-	    } else {
-		pack append $fm $fm.but$id {top fill expand}
-	    }
-	    tkwait window .config
-	    wm deiconify .
+	    installAlternates $files 1
 	}
     }
 }
-proc installConfig { {file {}} } {
+proc installAlternates { files isdefault } {
+    wm withdraw .
+    toplevel .config
+    set fm [frame .config.rim -bd 10]
+    message $fm.msg -aspect 1500 -text \
+"Please select an alternate configuration."
+    pack $fm -fill both -expand 1
+    pack $fm.msg
+    set id 0
+    foreach f $files {
+	button $fm.but$id -text $f -command [list installConfig $f $isdefault]
+	pack $fm.but$id -fill both -expand 1
+	incr id
+    }
+    button $fm.but$id -text "Ignore configurations" -command installConfig
+    pack $fm.but$id -fill both -expand 1
+    tkwait window .config
+    wm deiconify .
+}
+proc installConfigs {} {
+    global install
+    if {[catch {glob ../$install(appName)*/$install(dotFile)*} files] == 0} {
+	installAlternates $files 0
+    }
+}
+proc installConfig { {file {}} {isdefault 1} } {
+    global install
+    installFeedback "installConfig $file"
     if {$file != {}} {
 	if [catch {uplevel #0 source $file} msg] {
 	    installFeedback $msg
 	    return
 	}
+	if {! $isdefault} {
+	    set install(dotFile) $file
+	    installUpdateAll
+	}
     }
     destroy .config
 }
@@ -122,6 +132,12 @@ proc install_expect { var pathname comment } {
     set install(progcheck,$var) $comment
 }
 
+proc install_ps { var cmd comment } {
+    install_var $var $cmd $comment
+    global install
+    set install(psVar) $var
+}
+
 proc installFieldVar { item } { lindex $item 0 }
 proc installFieldComment { item } { lindex $item 2 }
 proc installFieldDefault { item {override 0} } {
@@ -165,7 +181,7 @@ proc installDoField { item {override 0} } {
     set var [installFieldVar $item]
     button $f.label -text [format "%-30s:" [installFieldComment $item]] \
 	-command "installShowValue $var" -font fixed
-    entry $f.entry -bg white -font fixed -width 35
+    entry $f.entry -font fixed -width 35
     $f.entry insert 0 [installFieldDefault $item $override]
     bind $f.entry <Return> [list installSetValue $var]
     if {$install(lastentry) != {}} {
@@ -177,9 +193,16 @@ proc installDoField { item {override 0} } {
     set install(entry,$var) $f.entry
     lappend install(allEntries) $f.entry
 
-    pack append .rim $f {top expand fill}
-    pack append $f $f.label {left padx 3}
-    pack append $f $f.entry {right expand fill}
+    pack $f -side top -expand true -fill both
+    pack $f.label -side left -padx 3
+    pack $f.entry -side right -expand true -fill both
+}
+proc installUpdateField { item {override 0} } {
+    global install
+    set var [installFieldVar $item]
+    set entry $install(entry,$var)
+    $entry delete 0 end
+    $entry insert 0 [installFieldDefault $item $override]
 }
 proc installFieldDone {} {
     global install
@@ -200,7 +223,7 @@ proc installSetValue { _var } {
 	    global $_var
 	}
 	set $_var $_value
-	installFeedback "$_var $_value"
+#	installFeedback "$_var $_value"
     }
 }
 proc installGetValue { var } {
@@ -230,6 +253,9 @@ proc installVerify {} {
 	foreach var $install(dircheck) {
 	    installSetValue $var
 	    set path [installGetValue $var]
+	    if {[string length $path] == 0} {
+		continue
+	    }
 	    if ![file isdirectory $path] {
 		set willMakeDir 0
 		foreach dirType $install(dirlist) {
@@ -253,6 +279,9 @@ proc installVerify {} {
 	foreach var $install(filecheck) {
 	    installSetValue $var
 	    set path [installGetValue $var]
+	    if {[string length $path] == 0} {
+		continue
+	    }
 	    if ![file exists $path] {
 		lappend errors [format "%-30s <%s> %s" \
 		    $install(filecheck,$var) $path "does not exist"]
@@ -267,6 +296,9 @@ proc installVerify {} {
 	foreach var $install(progcheck) {
 	    installSetValue $var	;# Snarf current value from entry
 	    set path [installGetValue $var]
+	    if {[string length $path] == 0} {
+		continue
+	    }
 	    if ![file executable $path] {
 		lappend errors [format "%-30s <%s> %s" \
 		    $install(progcheck,$var) $path "is not executable"]
@@ -277,6 +309,14 @@ proc installVerify {} {
 	    }
 	}
     }
+    if [info exists install(psVar)] {
+	installSetValue $install(psVar)	;# Snarf current value from entry
+	set cmd [installGetValue $install(psVar)]
+	if [catch {eval exec $cmd [pid]} err] {
+	    lappend errors [format "%-30s Warning: <%s> %s" \
+		"ps command" "$cmd [pid]" $err]
+	}
+    }
     if {$errors != {}} {
 	installFeedback "Verify errors"
     } else {
@@ -287,19 +327,19 @@ proc installVerify {} {
     frame .verify.top
     button .verify.top.quit -text "Dismiss" -command {destroy .verify}
     label .verify.top.label -text "  Verify Errors "
-    pack append .verify .verify.top {top fill expand}
-    pack append .verify.top .verify.top.quit left .verify.top.label {left fill}
+    pack .verify.top -side top -fill both -expand true
+    pack .verify.top.quit -side left
+    pack .verify.top.label -side left -fill both
 
     set numLines [llength $errors]
     if {$numLines < 30} {
 	text .verify.t -width 80 -height $numLines -font fixed
-	pack append .verify .verify.t {bottom expand fill}
+	pack .verify.t -side bottom -expand true -fill both
     } else {
 	text .verify.t -width 80 -height 30 -yscrollcommand {.verify.s set} -font fixed
 	scrollbar .verify.s -orient vert -command {.verify.t yview}
-	pack append .verify \
-	    .verify.s {right filly} \
-	    .verify.t {left expand fill}
+	pack .verify.s -side right -fill y
+	pack .verify.t -side left -expand true -fill both
     }
     foreach line $errors {
 	.verify.t insert end $line\n
@@ -318,14 +358,25 @@ proc installSed { } {
 	}
     }
     installSetValue wish	;# Get current value from entry widget
-    puts $script "s,#!wish,#![installGetValue wish],"
+    set w [installGetValue wish]
+    foreach sep {, ` ~ ? &} {
+	if {![string match *$sep* $w]} {
+	    break
+	}
+    }
+    puts $script "s$sep#!wish$sep#!$w$sep"
 
     # Set up for helper expect scripts, if needed and if possible
     if [info exists install(expectVar)] {
 	installSetValue $install(expectVar)
 	set pathname [installGetValue $install(expectVar)]
 	if {$pathname != {}} {
-	    puts $script "s,#!expect,#!$pathname,"
+	    foreach sep {, ` ~ ? &} {
+		if {![string match *$sep* $pathname]} {
+		    break
+		}
+	    }
+	    puts $script "s$sep#!expect$sep#!$pathname$sep"
 	}
     }
 
@@ -344,7 +395,7 @@ proc installSed { } {
     foreach prog $install(sedProgs) {
 	if [catch {
 	    exec sed -f /tmp/sed.$id < ${prog}$install(sedSuffix) > $prog
-	    exec chmod +x $prog
+	    exec chmod a+x $prog
 	} msg] {
 	    installFeedback "sed error on $prog: $msg"
 	}
@@ -368,19 +419,19 @@ proc installPatch {} {
     frame .test.top
     button .test.top.quit -text "Dismiss" -command {destroy .test}
     label .test.top.label -text "  Context diff of $sample"
-    pack append .test .test.top {top fill expand}
-    pack append .test.top .test.top.quit left .test.top.label {left fill}
+    pack .test.top -side top -fill both -expand true
+    pack .test.top.quit -side left 
+    pack .test.top.label -side left -fill both
 
     installFeedback "$numLines lines of diff output"
     if {$numLines < 30} {
 	text .test.t -width 80 -height $numLines -font fixed
-	pack append .test .test.t {bottom expand fill}
+	pack .test.t -side bottom -expand true -fill both
     } else {
 	text .test.t -width 80 -height 30 -yscrollcommand {.test.s set} -font fixed
 	scrollbar .test.s -orient vert -command {.test.t yview}
-	pack append .test \
-	    .test.s {right filly} \
-	    .test.t {left expand fill}
+	pack .test.s -side right -fill y
+	pack .test.t -side left -expand true -fill both
     }
     .test.t insert end $diff
 }
@@ -418,10 +469,10 @@ proc installTclIndex {} {
 }
 
 proc installButton {} {
-    button .rim.buttons.yes -text "Really Install" -command {installInner}
+	button .rim.buttons.yes -text "Really Install" -command {installInner}
     button .rim.buttons.no -text "Cancel" -command {installCancel}
-    pack unpack .rim.buttons.install
-    pack before .rim.buttons.quit .rim.buttons.no left .rim.buttons.yes left
+    pack forget .rim.buttons.install
+    pack .rim.buttons.no .rim.buttons.yes -side left
 }
 proc installSave { } {
     global install argv0
@@ -449,7 +500,7 @@ proc installSave { } {
 proc installCancel {} {
     after 10 {
 	destroy .rim.buttons.yes ; destroy .rim.buttons.no
-	pack before .rim.buttons.quit .rim.buttons.install left
+	pack .rim.buttons.install -before .rim.buttons.quit -side left
     }
 }
 proc installCmd { logProc unixCmd } {
@@ -468,9 +519,7 @@ proc installInner { {logProc nolog} } {
 	# Install directory - make sure it exists
 	#
 	set dir $install(dir,$dirType)
-	if ![file isdirectory $dir] {
-	    installCmd $logProc [list mkdir $dir]
-	}
+	MakeDir $logProc $dir
 	if {($logProc == "nolog") && ![file isdirectory $dir]} {
 	    installError "LibDir $dir is not a directory"
 	    continue
@@ -492,6 +541,11 @@ proc installInner { {logProc nolog} } {
 		    }
 		    installCmd $logProc [list rm -f $dir/$newf]
 		    installCmd $logProc [list cp $f $dir/$newf]
+		    if {$dirType == "bin"} {
+			installCmd $logProc [list chmod a+rx $dir/$newf]
+		    } else {
+			installCmd $logProc [list chmod a+r $dir/$newf]
+		    }
 		} msg] {
 		    installFeedback "Dir install error: $msg"
 		    return
@@ -508,20 +562,36 @@ proc installInner { {logProc nolog} } {
 	installFeedback "Install complete"
     }
 }
+proc MakeDir { logProc dir } {
+    if [file isdirectory $dir] {
+	return 1
+    } elseif [file exists $dir] {
+	installError "LibDir $dir is not a directory"
+	return 0
+    } else {
+	if [MakeDir $logProc [file dirname $dir]] {
+	    installCmd $logProc [list mkdir $dir]
+	    installCmd $logProc [list chmod a+rx $dir]
+	    return 1
+	} else {
+	    return 0
+	}
+    }
+}
 proc installFake {} {
     global exmh install
     toplevel .fake
     frame .fake.top
     button .fake.top.quit -text "Dismiss" -command {destroy .fake}
     label .fake.top.label -text "  Pending install actions"
-    pack append .fake .fake.top {top fill expand}
-    pack append .fake.top .fake.top.quit left .fake.top.label {left fill}
+    pack .fake.top -side top -fill both -expand true
+    pack .fake.top.quit -side left
+    pack .fake.top.label -side left -fill both
 
     text .fake.t -width 80 -height 20 -yscrollcommand {.fake.s set} -font fixed
     scrollbar .fake.s -orient vert -command {.fake.t yview}
-    pack append .fake \
-	.fake.s {right filly} \
-	.fake.t {left expand fill}
+    pack .fake.s -side right -fill y
+    pack .fake.t -side left -expand true -fill both
 
     proc log { text } {
 	.fake.t insert end $text\n
@@ -529,20 +599,28 @@ proc installFake {} {
     installInner log
 }
 proc install_dialog {} {
-    global install
+    global install tk_version
+
+    wm title . "Exmh Install"
+    wm minsize . 200 200
+
     toplevel .info
     wm title .info "Install info for $install(appName)"
 
-    wm minsize . 100 100
-    
-    message .info.msg -aspect 1000 -font fixed -text $install(helpText)
-    pack append .info .info.msg {top expand fill}
+
+    text .info.t
+    .info.t config -yscrollcommand {.info.s set} -setgrid true
+    wm minsize .info 40 10
+    scrollbar .info.s -orient vert -command {.info.t yview}
+    pack .info.s -side right -fill y
+    pack .info.t -side left -fill both -expand true
+    .info.t insert 1.0 $install(helpText)
     update idletasks
 
     installBindInit
     
     frame .rim -bd 5 -relief flat
-    pack append . .rim {top expand fill}
+    pack .rim -side top -expand true -fill both
 
     installFieldInit
     foreach v $install(sequence) {
@@ -555,14 +633,15 @@ proc install_dialog {} {
     }
     installFieldDone
 
-    set install(msg) [label .rim.feedback -text ""]
-    pack append .rim $install(msg) {top expand fill}
+    set install(msg) [label .rim.feedback -text "" -anchor w -padx 10]
+    pack $install(msg) -side top -expand true -fill both
     
     frame .rim.buttons -relief raised
-    pack append .rim .rim.buttons {top expand fill}
+    pack .rim.buttons -side top -expand true -fill both
     
-    button .rim.buttons.quit -text "Quit" -command {destroy .}
+    button .rim.buttons.quit -text "Quit" -command {exit}
     button .rim.buttons.keys -text "Keys" -command {installBindKeys}
+    button .rim.buttons.conf -text "Conf" -command {installConfigs}
     button .rim.buttons.patch -text "Patch" -command {installPatch}
     button .rim.buttons.test -text "Test" -command {installTest}
     button .rim.buttons.verify -text "Verify" -command {installFake}
@@ -570,15 +649,34 @@ proc install_dialog {} {
     button .rim.buttons.tclindex -text "TclIndex" -command {installTclIndex}
     frame .rim.buttons.space -width 10 -height 10
 
-    pack append .rim.buttons \
-	.rim.buttons.patch left \
-	.rim.buttons.tclindex left \
-	.rim.buttons.test left \
-	.rim.buttons.verify left \
-	.rim.buttons.space left \
-	.rim.buttons.install left \
-	.rim.buttons.quit right \
-	.rim.buttons.keys right
+    button .rim.buttons.readme -text "I have read the instructions" \
+	-command install_pack_buttons
+    pack .rim.buttons.readme -side left
+    pack .rim.buttons.quit -side right
+}
+proc install_pack_buttons {} {
+    pack forget .rim.buttons.readme
+    pack .rim.buttons.patch  \
+	.rim.buttons.tclindex  \
+	.rim.buttons.test  \
+	.rim.buttons.verify  \
+	.rim.buttons.space  \
+	.rim.buttons.install -side left
+    pack \
+	.rim.buttons.quit  \
+	.rim.buttons.keys  \
+	.rim.buttons.conf -side right
+}
+proc installUpdateAll {} {
+    global install
+    foreach v $install(sequence) {
+	if {$v == $install(versionVar)} {
+	    set override 1	;# over-ride saved value with new version num.
+	} else {
+	    set override 0
+	}
+	installUpdateField $install(field,$v) $override
+    }
 }
 proc installBindInit {} {
     global install
@@ -607,11 +705,7 @@ proc installBindKeys {} {
     toplevel .keys
     wm title .keys "Install Edit Preferences"
     frame .keys.b
-    if {$tk_version >= 3.3} {
-	pack .keys.b -side top -fill x
-    } else {
-	pack append .keys .keys.b {top fill}
-    }
+    pack .keys.b -side top -fill x
     button .keys.b.quit -text Dismiss -command {destroy .keys}
     button .keys.b.apply -text Apply -command installBindSet
 
@@ -619,13 +713,8 @@ proc installBindKeys {} {
 "Key bindings for the installation tool.
 Changes *do not* carry over to $install(appName)"
     set f [frame .keys.p -bd 10]
-    if {$tk_version >= 3.3} {
-	pack .keys.b .keys.msg .keys.p -side top -fill x
-	pack .keys.b.quit .keys.b.apply -side right
-    } else {
-	pack append .keys .keys.b {top fill} .keys.msg {top fill} .keys.p {top fill}
-	pack append .keys.b .keys.b.quit right .keys.b.apply right
-    }
+    pack .keys.b .keys.msg .keys.p -side top -fill x
+    pack .keys.b.quit .keys.b.apply -side right
 
     set width 0
     foreach item [array names install] {
@@ -644,21 +733,16 @@ Changes *do not* carry over to $install(appName)"
     }
 }
 proc installKeyItem { frame width name keystroke } {
-    global install tk_version
+    global install
     frame $frame.$name
     label $frame.$name.label -text $name -width $width
-    entry $frame.$name.entry -background white
+    entry $frame.$name.entry 
     set install(keyval,$name) $frame.$name.entry
     $frame.$name.entry insert 0 $keystroke
     lappend install(allEntries) $frame.$name.entry
-    if {$tk_version >= 3.3} {
-	pack $frame.$name
-	pack $frame.$name.label -side left
-	pack $frame.$name.entry -side right -fill x -expand 1
-    } else {
-	pack append $frame $frame.$name {top expand fill}
-	pack append $frame $frame.label left $frame.entry {right fillx expand}
-    }
+    pack $frame.$name
+    pack $frame.$name.label -side left
+    pack $frame.$name.entry -side right -fill x -expand 1
 }
 proc installBindSet {} {
     global install
@@ -687,7 +771,7 @@ proc installBindEntry { {list {}} } {
     }
 }
 proc installBindEntryInner { what } {
-    global install
+    global install tk_version
 
     # Modification bindings
 
@@ -703,10 +787,13 @@ proc installBindEntryInner { what } {
 	catch {%W delete sel.first sel.last}
     }
 
+    if {$tk_version >= 4.0} {
+	set bsProc tkEntryBackspace
+    } else {
+	set bsProc tk_entryBackspace
+    }
     foreach bs {backspace backspace2 backspace3} {
-	bind $what $install(key,$bs) {
-	    tk_entryBackspace %W
-	}
+	bind $what $install(key,$bs) "$bsProc %W"
     }
 
     bind $what $install(key,deleol) {
