@@ -75,10 +75,11 @@ proc Aliases_Pref { {nalias {}} } {
 		[list .aliaspref.alias.entry Entry SearchAlias]
 	    bind SearchAlias <KeyPress> \
 		 {Widget_ListSearch .aliaspref.alias}
-	    bind .aliaspref.alias.entry <space> \
-		{set alias [AliasesComplete .aliaspref.alias]
-		 Aliases_Insert
-		 break}
+	    bind .aliaspref.alias.entry <space> {
+		set alias [AliasesComplete .aliaspref.alias]
+		Aliases_Insert
+		break
+	    }
 	    bind .aliaspref.alias.entry <Control-c> \
 		{Aliases_Delete ; break}
 
@@ -87,8 +88,8 @@ proc Aliases_Pref { {nalias {}} } {
 	    bind SearchAddress <KeyPress> \
 		 {Widget_ListSearch .aliaspref.addr}
 	    bind .aliaspref.addr.entry <Control-space> {
-		{set address [AliasesComplete .aliaspref.addr]
-		 break}
+		set address [AliasesComplete .aliaspref.addr]
+		break
 	    }
 	    foreach e {.aliaspref.alias.entry .aliaspref.addr.entry} {
 		Exmh_Debug bindtags $e [bindtags $e]
