@@ -16,10 +16,19 @@
 #	from commands bound to keystrokes and menus.  A
 #	toplevel message widget is used to display $errorInfo
 
-proc tkerror { msg } {
-    global errorInfo
-    global exmh
+set ErrorIgnore {
+    {^grab failed}
+}
 
+proc tkerror { msg } {
+    global errorInfo exmh ErrorIgnore
+
+    foreach pat $ErrorIgnore {
+	if [regexp $pat $msg] {
+	    Exmh_Status $msg
+	    return
+	}
+    }
     set font fixed
     set base ".errorInfo"
     set title "Error Info"
@@ -104,11 +113,12 @@ pressing the Mail to button, or just Dismiss me."
 	puts stderr $savedErrorInfo
     }
 }
+proc bgerror [info args tkerror] [info body tkerror]
 
 proc ExmhMailError { w errInfo } {
     global exmh
-    if [catch {open /tmp/exmhErrorMsg w} out] {
-	Exmh_Status "Cannot open /tmp/exmhErrorMsg" purple
+    if [catch {open [Env_Tmp]/exmhErrorMsg w} out] {
+	Exmh_Status "Cannot open [Env_Tmp]/exmhErrorMsg" purple
 	return
     }
     if [catch {
@@ -137,11 +147,11 @@ proc ExmhMailError { w errInfo } {
 	puts $out $errInfo
 	close $out
     } msg] {
-	Exmh_Status "/tmp/exmhErrorMsg $msg" purple
+	Exmh_Status "[Env_Tmp]/exmhErrorMsg $msg" purple
 	return
     }
     if [catch {
-	exec send /tmp/exmhErrorMsg
+	exec send [Env_Tmp]/exmhErrorMsg
     } msg] {
 	Exmh_Status "Send error: $msg" purple
 	return
@@ -203,6 +213,13 @@ proc Send_Error { msg draftID } {
 	    EditWhatNow $draftID
 	"
     pack append $name.buttons $name.buttons.edit {right}
+
+    button $name.buttons.retry -text "Retry Send" -command "
+	    destroy $name
+            unset $name
+	    Edit_Done send $draftID
+	"
+    pack append $name.buttons $name.buttons.retry {right}
     global widgetText TextType
 
     text $name.msg -font $font -width 60 -bd 2 -relief raised
