@@ -23,6 +23,9 @@
 # c_moved	$fdisp(c_moved)		Target folder / moved message
 # c_deleted	$fdisp(c_deleted)	Deleted message
 #
+# For monochrome displays, the Canvas.foreground and Canvas.background
+# resources determine the foreground and background colors of the labels
+#
 # For consistency these colors are shared with the folder table of contents
 # display.  If you find this limiting, wack away on the itemconfigures...
 #
@@ -44,21 +47,23 @@ proc Fdisp_LabelConfigure { canvas } {
     # 
     Fdisp_FixupSpecials $canvas
     if {[tk colormodel .] == "monochrome"} {
-	$canvas itemconfigure text -fill black
-	$canvas itemconfigure box -fill white -width 1 -stipple {}
-	$canvas itemconfigure bitmap -foreground black -background white
+	set fg [option get . c_foreground {}]
+	set bg [option get . c_background {}]
+	$canvas itemconfigure text -fill $fg
+	$canvas itemconfigure box -fill $bg -width 1 -stipple {} -outline $fg
+	$canvas itemconfigure bitmap -foreground $fg -background $bg
 
-	$canvas itemconfigure unsntext -fill black
-	$canvas itemconfigure unsnbox -width 3
-	$canvas itemconfigure unsnbitmap -foreground white -background black
+	$canvas itemconfigure unsntext -fill $fg
+	$canvas itemconfigure unsnbox -width 3 -outline $fg
+	$canvas itemconfigure unsnbitmap -foreground $bg -background $fg
 
-	$canvas itemconfigure tartext -fill black
-	$canvas itemconfigure tarbox -fill black -stipple gray25
-	$canvas itemconfigure tarbitmap -foreground white -background black
+	$canvas itemconfigure tartext -fill $fg
+	$canvas itemconfigure tarbox -fill $fg -stipple gray25
+	$canvas itemconfigure tarbitmap -foreground $bg -background $fg
 
-	$canvas itemconfigure curtext -fill white
-	$canvas itemconfigure curbox -fill black -stipple {}
-	$canvas itemconfigure curbitmap -foreground white -background black
+	$canvas itemconfigure curtext -fill $bg
+	$canvas itemconfigure curbox -fill $fg -stipple {}
+	$canvas itemconfigure curbitmap -foreground $bg -background $fg
     } else {
 	$canvas itemconfigure text -fill $fdisp(c_fg)
 	$canvas itemconfigure box -fill $fdisp(c_bg) \
@@ -86,9 +91,29 @@ proc Fdisp_LabelConfigure { canvas } {
     }
 }
 
-proc FdispClearHighlights { canvas } {
-    foreach tag {unsnbitmap unsnbox unsntext} {
+proc FdispClearHighlights { can } {
+    global fdisp
+
+    set canvas $fdisp($can)
+
+    foreach tag [concat {unsnbitmap unsnbox unsntext} $fdisp(leafs,$can)] {
 	$canvas dtag $tag
     }
+    # The leaf tags cause ancestor nodes to be highlighted
+    set fdisp(leafs,$can) {}
+
+    # Deleting tags does not undo looks (like text widget!)
+    # So we manually reset looks on the labels here.
+    if {[tk colormodel .] == "monochrome"} {
+	$canvas itemconfigure text -fill black
+	$canvas itemconfigure box -fill white -width 1 -stipple {}
+	$canvas itemconfigure bitmap -foreground black -background white
+    } else {
+	$canvas itemconfigure text -fill $fdisp(c_fg)
+	$canvas itemconfigure box -fill $fdisp(c_bg) \
+				-outline $fdisp(c_fg) -width 1 -stipple {}
+	$canvas itemconfigure bitmap -foreground $fdisp(c_fg) \
+				-background $fdisp(c_bg)
+    }
 }
 
