@@ -158,6 +158,14 @@ proc Folder_TargetCopy { f {copyProc Ftoc_CopyMark} } {
     Msg_Move $copyProc advance?
 }
 
+proc Folder_TargetClear {f} {
+    global exmh
+
+    Fdisp_HighlightTarget ""
+    set exmh(target) ""
+    Exmh_Status "No target set for moves"
+}
+
 
 proc Folder_Sort { args } {
     global exmh
