@@ -11,27 +11,29 @@
 # any specification.
 
 proc FolderDialogInner {path label cancelProc okProc message} {
-    global mhProfile exwin exmh
+    global mhProfile exwin exmh tk_version
 
-    if [Exwin_Toplevel $path $label FolderDialog nomenu] {
-	set t $path
+   set t $path
+   if [Exwin_Toplevel $path $label FolderDialog nomenu] {
 	wm transient $t
-    
+
 	Widget_Message $t msg -aspect 400 -text $message
 	set f [Widget_Frame $t name Rim]
 	$f configure -bd 10
-	set f [Widget_Frame $f rim Rim]
-	$f configure -bd 2 -relief raised
+	set f [Widget_Frame $f rim LabeledEntry]
 	Widget_Label $f label {left} -text "Folder Name:"
-	set e [Widget_Entry $f name {right fill} -bg white -relief raised]
-	bind $e <Return> [list $okProc $path $e]
-	focus $e
-    
+	set e [Widget_Entry $f name {right fill}]
+	Widget_BindEntryCmd $e <Return> "$okProc $path $e"
+
 	set f [Widget_Frame $t but Rim]
 	$f configure -bd 10
 	Widget_AddBut $f cancel "Cancel" [list $cancelProc $path] left
 	Widget_AddBut $f ok "OK" [list $okProc $path $e] right
+    } else {
+	$t.msg config -text $message
+	set e $path.name.rim.name
     }
+    focus $e
 }
 
 proc Folder_New {} {
@@ -40,19 +42,27 @@ proc Folder_New {} {
     FolderDialogInner .newf "Create Folder" Exwin_Dismiss FolderNewCommit \
 "Creating a new folder results in a new directory under $mhProfile(path).
 Nested folders are allowed.
-You need NOT put a + after the folder name.
+You need NOT put a + before the folder name.
 @ is shorthand for the current folder, so
 @foo => $exmh(folder)/foo"
 }
 
 proc FolderNewCommit { top entry } {
     global mhProfile exmh
-    set name [$entry get]
-    if [regexp { } $name] {
-	Exmh_Status "NO SPACES in folder names" warn
+    set name [string trim [$entry get]]
+    if {[string length $name] == 0} {
+	Exmh_Status "Please enter a folder name"
 	return
     }
-    if [regexp {@(.*)} $name all newname] {
+    if [regexp { } $name] {
+	Exmh_Status "NO SPACES in folder names" error
+	return
+    }
+    if [regexp {^[0-9]+$} $name] {
+	Exmh_Status "No pure NUMBERS for folder names" error
+	return	
+    }
+    if [regexp {^@(.*)} $name all newname] {
 	set name $exmh(folder)
 	if {[string length $newname] > 0} {
 	    set name $name/$newname
@@ -81,6 +91,7 @@ proc FolderNewCommit { top entry } {
     }
     Exmh_Status "Created folder $name" blue
     Flist_AddFolder $name
+    Fcache_Folder $name
     Exwin_Dismiss $top
     return
 }
@@ -90,15 +101,19 @@ proc Folder_Delete {} {
 
     FolderDialogInner .delf "Delete Folder" Exwin_Dismiss FolderDelCommit \
 "Only folders with no messages can be deleted.
-You need NOT put a + after the folder name.
+You need NOT put a + before the folder name.
 @ is shorthand for the current folder, so
 @foo => $exmh(folder)/foo"
 }
 
 proc FolderDelCommit { top entry } {
     global mhProfile exmh
-    set name [$entry get]
-    if [regexp {@(.*)} $name all newname] {
+    set name [string trim [$entry get]]
+    if {[string length $name] == 0} {
+	Exmh_Status "Please enter a folder name"
+	return
+    }
+    if [regexp {^@(.*)} $name all newname] {
 	set name $exmh(folder)
 	if {[string length $newname] > 0} {
 	    set name $name/$newname
@@ -109,6 +124,7 @@ proc FolderDelCommit { top entry } {
 	Exmh_Status "Cleaning up folder $name" blue
 	Flist_DelFolder $name
 	Fcache_FolderDiscard $name
+	Glimpse_Delete $name
 	Exwin_Dismiss $top
 	return
     }
@@ -125,6 +141,7 @@ proc FolderDelCommit { top entry } {
 	    Exmh_Status "Deleted folder $name" blue
 	    Flist_DelFolder $name
 	    Fcache_FolderDiscard $name
+	    Glimpse_Delete $name
 	}
     } else {
 	Exmh_Status "Still messages in +$name"
