@@ -465,6 +465,7 @@ proc Msg_BurstDigest {} {
 		lappend allids $msgid
 	    }
 	}
+	set allids [lsort -increasing -integer $allids]
 	# mark new messages as unread
 	Exmh_Debug burst created msgs $allids
 	if {$allids != {}} {
@@ -509,6 +510,10 @@ proc Msg_Save {} {
 
 proc Msg_Edit {} {
     global exmh msg editor
+    if {$msg(path) == ""} {
+	Exmh_Status "No current message"
+	return
+    }
     Exmh_Status "Editing $exmh(folder)/$msg(id)"
     #
     # Hack because exmh-async isn't appropriate in this case.
@@ -557,7 +562,11 @@ proc Msg_ReplyHelp {} {
 
 proc Msg_PageOrNext {} {
     global exwin
-    Widget_TextPageOrNext $exwin(mtext)
+    Widget_TextPageOrNext $exwin(mtext) implied
+}
+proc Msg_PageOrNextCommit {} {
+    global exwin
+    Widget_TextPageOrNext $exwin(mtext) no
 }
 proc Msg_PageDown {} {
     global exwin
