@@ -22,13 +22,6 @@ proc Pick {} {
 	Widget_CheckBut $f add "Add to Sel" pick(addtosel) {left padx 1}
 	Widget_AddBut $f project "New FTOC" Ftoc_NewFtoc {left padx 1}
 
-	global tk_version
-	if {$tk_version >= 4.0} {
-	    # So our own <Tab> and <Shift-Tab> bindings skip the
-	    # default action to enter the character.
-	    bind .pick <Tab> break
-	    bind .pick <Shift-Tab> break
-	}
 	PickSetup
         Widget_AddBut .pick or "-Or-" PickOr {bottom pady 5}
     }
