@@ -34,6 +34,9 @@ proc Text_SelectTo {w index} {
      if [catch {$w index anchor}] {
 	return
      }
+    if ![info exists tk_priv(selectMode)] {
+	set tk_priv(selectMode) char
+    }
     case $tk_priv(selectMode) {
 	char {
 	    if [$w compare $index == anchor] {
@@ -133,10 +136,12 @@ proc Text_StartExtend { w index } {
 
 # Return the current selection (from any window) or cut buffer 0.
 proc Text_Selection {} {
-    set sel ""
-    catch {set sel [selection get]}
-    if {[string length $sel] == 0} {
-	catch {set sel [cutbuffer get 0]}
+    if [catch {selection get} sel] {
+	if [catch {selection get -selection CLIPBOARD} sel] {
+	    if [catch {cutbuffer get 0} sel] {
+		return ""
+	    }
+	}
     }
     return $sel
 }
@@ -151,6 +156,7 @@ proc Text_Delete {w start {end {}} {addkill 0}} {
 	set TextType($w) text
     }
     set st [$w index $start]
+    set tags [$w tag names $st]		;# Hack to save tags
     if {[string length $end] == 0} {
 	set e [$w index "$start + 1c"]
     } else {
@@ -181,6 +187,10 @@ proc Text_Delete {w start {end {}} {addkill 0}} {
 	    $w delete $st $e
 	}
     }
+    # This hack supports the tags used in sedit to label parts of multiparts
+    foreach tag $tags {
+	$w tag add $tag $st "$st lineend +1c"
+    }
 }
 
 # Do kill buffer processing.  If addkill is true, then add to the kill ring
@@ -212,6 +222,7 @@ proc Text_DoKill {w start end addkill} {
 	set tk_priv(lastsel) $text
     }
     cutbuffer set 0 $tk_priv(lastsel)
+    catch {clipboard clear ; clipboard append $tk_priv(lastsel)}
     selection own $w "Text_LoseSelection $w"
     set tk_priv(delstate) "$w killing $start"
 }
@@ -281,9 +292,13 @@ proc Text_Insert {w place text} {
 }
 
 proc Text_Yank { w } {
-    global tk_priv
+    global tk_priv sedit
     set sel [Text_Selection]
     if {[string length $sel] != 0} {
+	# check for 8bit characters in the selection
+	if [regexp "\[\x80-\xff\]" $sel] {
+	    set sedit($w,8bit) 1
+	}
 	set start [$w index insert]
 	Text_Insert $w insert $sel
 	$w yview -pickplace insert
@@ -310,12 +325,12 @@ proc Text_YankPop { w } {
     set tk_priv(delstate) "$w yank $start $end"
 }
 
-proc Text_MoveInsert {w place} {
+proc Text_MoveInsert {w place {clear clear}} {
     global tk_priv
     set tk_priv(selectMode) char
     set tk_priv(delstate) {}
     global sedit
-    if {$sedit(typeKillsSel)} {
+    if {[string compare $clear "clear"] == 0 && $sedit(typeKillsSel)} {
         $w tag remove sel 0.0 end
     }
     $w mark set insert $place
@@ -362,11 +377,15 @@ proc Text_NextWord {w index} {
 
 proc Text_KillSelection { w } {
     global tk_priv
-    set res [expr {! [catch {
-	Text_Delete $w sel.first sel.last
-    }]}]
-    $w tag remove sel 0.0 end
-    return $res
+    return [expr ! [catch {
+	if {[$w compare sel.first <= insert] &&
+	    [$w compare sel.last >= insert]} {
+	    Text_Delete $w sel.first sel.last 1
+	    $w tag remove sel 0.0 end
+	} else {
+	    error "cursor outside selection"
+	}
+    }]]
 }
 
 proc Text_Backspace w {
@@ -431,6 +450,17 @@ proc Text_TransposeChars w {
 	set c [$w get insert-1c insert]
 	Text_Delete $w insert-1c insert
 	Text_Insert $w insert-1c $c
+
+    }
+}
+
+proc Text_TransposeCharsEmacs w {
+    if {[$w compare insert >= "[Text_GetFence $w] + 1c"]} {
+	set c [$w get insert-1c insert]
+	Text_Delete $w insert-1c insert
+	Text_Insert $w insert+1c $c
+	Text_MoveInsert $w insert+2c
+
     }
 }
 
